#!/usr/bin/python3

from ..core import AbstractQObject
from PySide6.QtWidgets import (
    QWidget,
    QGraphicsBlurEffect,
    QGraphicsColorizeEffect,
    QGraphicsDropShadowEffect,
    QGraphicsOpacityEffect
)


class BlurEffect(AbstractQObject, QGraphicsBlurEffect):
    def __init__(self,  **kwargs) -> QWidget:
        QGraphicsBlurEffect.__init__(self)
        super().__init__(**kwargs)


class ColorizeEffect(AbstractQObject, QGraphicsColorizeEffect):
    def __init__(self, **kwargs) -> QWidget:
        QGraphicsColorizeEffect.__init__(self)
        super().__init__(**kwargs)


class DropShadowEffect(AbstractQObject, QGraphicsDropShadowEffect):
    def __init__(self, **kwargs):
        QGraphicsDropShadowEffect.__init__(self)
        super().__init__(**kwargs)


class OpacityEffect(AbstractQObject, QGraphicsOpacityEffect):
    def __init__(self, **kwargs) -> QWidget:
        QGraphicsOpacityEffect.__init__(self)
        super().__init__(**kwargs)


#!/usr/bin/python3

from PySide6.QtGui import QColor
from ..core import AbstractTool
from ..enums.colors import Colors


class Color(AbstractTool, QColor):
    def __init__(self, *color, **kwargs):
        if color.__len__() and isinstance(color[0], Colors):
            color = (color.value,)

        QColor.__init__(self, *color)
        super().__init__(**kwargs)



from typing import Union
from PySide6.QtGui import QIcon, QPixmap
from ...core import AbstractDec


class MenuSeparatorWrapper:
    ...


class MenuSectionWrapper(AbstractDec):
    def __init__(self,
                 text: str,
                 icon: Union[QIcon, QPixmap] = None):

        return (text, icon) if icon is not None else (text,)



#!/usr/bin/python3

from PySide6.QtGui import QPaintEvent
from PySide6.QtWidgets import QWidget
from ._declarative import AbstractDec


class AbstractPainter(AbstractDec):
	def __init__(self, child: QWidget, **kwargs) -> QWidget:
		self._parent = child
		self._parent.paintEvent = self.paint

		return self._parent

	@property
	def _super(self) -> object:
		return self._parent.__class__

	def super_paintEvent(self, event: QPaintEvent) -> None:
		self._super.paintEvent(self._parent, event)

	def _paintEvent(self, event: QPaintEvent) -> None:
		self.super_paintEvent(event)

	def update(self) -> None:
		self.repaint()

	def repaint(self) -> None:
		self._parent.update()

	def paint(self, event: QPaintEvent) -> None:
		raise NotImplementedError


#!/usr/bin/python3

from typing import Union
from PySide6.QtGui import QAction
from PySide6.QtWidgets import QMenu
from ..tools.wrappers.menu import (
    MenuSectionWrapper, 
    MenuSeparatorWrapper
)
from ..tools import MenuAction
from ..core import AbstractWidget


class Menu(AbstractWidget, QMenu):
    def __init__(self, 
                 *,
                 children: list[Union[MenuSeparatorWrapper, 
                                      MenuSectionWrapper, 
                                      MenuAction,
                                      QMenu, 
                                      QAction]],
                 **kwargs):
        QMenu.__init__(self)
        super().__init__(**kwargs)

        if children is not None:
            for child in children:
                if isinstance(child, MenuSeparatorWrapper):
                    self.addSeparator()

                elif isinstance(child, MenuSectionWrapper):
                    self.addSection(*child)

                elif isinstance(child, (QAction, MenuAction)):
                    child.setParent(self)
                    self.addAction(child)

                elif isinstance(child, QMenu):
                    child.setParent(self)
                    self.addMenu(child)


#!/usr/bin/python3

from dataclasses import dataclass
from PySide6.QtCore import Qt
from PySide6.QtWidgets import QToolBar


@dataclass
class ToolBarWrapper:
    area: Qt.ToolBarArea
    toolbar: QToolBar

@dataclass
class ToolBarBreakWrapper:
    area: Qt.ToolBarArea = None



from typing import Union
from PySide6.QtCore import QIODevice
from PySide6.QtUiTools import QUiLoader
from PySide6.QtWidgets import QWidget
from ..core import QObjectDec
import os


class UiLoader(QObjectDec, QUiLoader):
    '''
    UiLoader(
        ui="/path/to/file.ui",
        ...
    ) # -> QWidget
    '''

    def __init__(self, ui: Union[str, bytes, os.PathLike, QIODevice], **kwargs) -> QWidget:
        QUiLoader.__init__(self)
        super().__init__(**kwargs)

        return self.load(ui)


#!/usr/bin/python3

from typing import Union, Callable
from PySide6.QtWidgets import QLineEdit
from PySide6.QtGui import QAction, QIcon, QPixmap
from ..core import AbstractQObject, QObjectDec


class LinePasswordAction(QObjectDec, QAction):
    def __init__(self,
                 child: QLineEdit,
                 show_icon: Union[QIcon, QPixmap] = None,
                 hide_icon: Union[QIcon, QPixmap] = None,
                 position: QLineEdit.ActionPosition = QLineEdit.ActionPosition.TrailingPosition,
                 **kwargs) -> QLineEdit:
        super().__init__(**kwargs)

        self._child = child
        self._show_icon = show_icon
        self._hide_icon = hide_icon
        self._position = position

        self.triggered.connect(lambda: self._toggle_password_echo_mode_icon())
        self._child.textChanged.connect(lambda: self._update_password_mode())
        self._child.addAction(self, self._position)

        self._toggle_icon()
        self._update_password_mode()

        return self._child

    def _toggle_password_echo_mode_icon(self):
        self._toggle_echo_mode()
        self._toggle_icon()

    def _toggle_echo_mode(self):
        self._child.setEchoMode(
            QLineEdit.EchoMode.Normal 
            if self._child.echoMode() == QLineEdit.EchoMode.Password
            else QLineEdit.EchoMode.Password
        )

    def _toggle_icon(self):
        self.setIcon(
            self._show_icon 
            if self._child.echoMode() == QLineEdit.EchoMode.Password 
            else self._hide_icon
        )

    def _update_password_mode(self):
        self.setVisible(bool(self._child.text()))

    def set_position(self, position):
        self._position = position

    def toggle_show_icon(self):
        self.setIcon(self._show_icon)

    def toggle_hide_icon(self):
        self.setIcon(self._hide_icon)


class MenuAction(AbstractQObject, QAction):
    def __init__(self,
                 text: str = None,
                 icon: Union[QIcon, QPixmap] = None,
                 callback: Callable = None,
                 **kwargs):
        QAction.__init__(self)

        if text is not None:
            self.setText(text)

        if icon is not None:
            self.setIcon(icon)

        if callback is not None:
            self.triggered.connect(callback)

        super().__init__(**kwargs)


from typing import Union
from PySide6.QtGui import QIcon, QIconEngine, QImage, QPixmap, QPainter, QColor
from PySide6.QtWidgets import QFileIconProvider
from PySide6.QtCore import QFileInfo, QSize, Qt
from ..core import AbstractTool, AbstractIcons


class Icon(AbstractTool, QIcon):
    def __init__(self,
                 icon: Union[str,
                             QIcon,
                             QIconEngine,
                             QPixmap,
                             QImage,
                             AbstractIcons],
                 color: QColor = None,
                 size: Union[QSize, tuple, int] = None,
                 **kwargs):

        if isinstance(icon, AbstractIcons):
            icon = icon.value

        if color is not None or size is not None:
            icon = self.colored_icon(icon, color, size)

        QIcon.__init__(self, icon)
        super().__init__(**kwargs)

    @classmethod
    def colored_icon(cls,
                     icon,
                     color: QColor,
                     size: Union[QSize, tuple, int] = None) -> QIcon:

        if isinstance(size, int):
            size = QSize(size, size)

        if isinstance(size, (tuple, list, set)):
            size = QSize(*size[:2])

        icon = QIcon(icon)
        default_size = (icon.availableSizes()[0] 
                        if len(icon.availableSizes()) > 0 
                        else icon.actualSize(QSize(32, 32)))

        pixmap = icon.pixmap(size 
                             if size is not None 
                             else default_size)

        pixmap_painter = QPainter(pixmap)
        pixmap_painter.setCompositionMode(QPainter.CompositionMode.CompositionMode_SourceIn)

        if color is not None:
            pixmap_painter.fillRect(pixmap.rect(), color if color is not None else 0)

        pixmap_painter.end()
        return QIcon(pixmap)

    @classmethod
    def make_svg(cls, 
                 svg: str,
                 color: QColor = None) -> QIcon:

        image = QImage()
        image.fill(Qt.GlobalColor.transparent)
        image.loadFromData(bytes(svg, "utf-8"))
        pixmap = QPixmap.fromImage(image, Qt.ImageConversionFlag.NoFormatConversion)

        if color is not None:
            pixmap_painter = QPainter(pixmap)
            pixmap_painter.setCompositionMode(QPainter.CompositionMode.CompositionMode_SourceIn)
            pixmap_painter.fillRect(pixmap.rect(), color if color is not None else -1)
            pixmap_painter.end()

        return QIcon(pixmap)

    @classmethod
    def file_provider(cls, icon: Union[str, QFileInfo]) -> QIcon:
        '''
        param: icon = [file.ext, /path/to/file.ext]
        '''
        return QFileIconProvider().icon(QFileInfo(icon) 
                                        if isinstance(icon, str) 
                                        else icon)


#!/usr/bin/python3
# -*- coding: utf-8 -*-

from PySide6.QtWidgets import QSpacerItem, QSizePolicy
from ..core import AbstractTool


class SpacerItem(AbstractTool, QSpacerItem):
    Policy = QSizePolicy.Policy
    def __init__(self, *args, **kwargs):
        QSpacerItem.__init__(self, *args)
        super().__init__(**kwargs)


from .menu import MenuSeparatorWrapper, MenuSectionWrapper
from .tool_bar import ToolBarBreakWrapper, ToolBarWrapper
from .h_layout import HLayoutWrapper, ColumnLayoutWrapper
from .v_layout import RowLayoutWrapper, VLayoutWrapper
from .border_layout import BorderLayoutWrapper
from .dock_widgets import DockWidgetWrapper
from .grid_layout import GridLayoutWrapper
from .form_layout import FormLayoutWrapper


#!/usr/bin/python3
# -*- coding: utf-8 -*-

from typing import Union
from PySide6.QtCore import Signal
from PySide6.QtGui import QShowEvent, QCloseEvent
from PySide6.QtWidgets import (
    QMainWindow,
    QWidget, 
    QToolBar,
    QDockWidget,
    QLayout
)
from ...tools.wrappers import (
    DockWidgetWrapper, 
    ToolBarBreakWrapper, 
    ToolBarWrapper
)


class BaseWindow(QMainWindow):
    startup_changed = Signal()

    def __init__(self, 
                 *,
                 home: Union[QWidget, QLayout] = None,
                 toolbars: list[ToolBarWrapper, ToolBarBreakWrapper, QToolBar, str] = None,
                 dockwidgets: list[DockWidgetWrapper] = None,
                 **kwargs):
        QMainWindow.__init__(self)

        self.setAnimated(True)
        self.setUpdatesEnabled(True)

        self.__is_startup = False

        if home is not None:
            self._set_home(home)
        
        if toolbars is not None:
            self._set_toolbars(toolbars)

        if dockwidgets is not None:
            self._set_dockwidgets(dockwidgets)

    def closeEvent(self, event: QCloseEvent) -> None:
        super().closeEvent(event)
        self.__is_startup = False

    def showEvent(self, event: QShowEvent) -> None:
        super().showEvent(event)
        if not self.__is_startup:
            self.__is_startup = True
            self.startup_changed.emit()

    def _set_home(self, home) -> None:
        if isinstance(home, QLayout):
            self.centralwidget = QWidget(self)
            self.centralwidget.setObjectName("centralwidget")
            home.setProperty("parent", self.centralwidget)
            self.centralwidget.setLayout(home)
            self.setCentralWidget(self.centralwidget)
        elif isinstance(home, QWidget):
            home.setParent(self)
            self.setCentralWidget(home)

    def _set_toolbars(self, toolbars) -> None:
        for bar in toolbars:
            if isinstance(bar, ToolBarWrapper):
                self.addToolBar(*[bar.area, bar.toolbar])
            elif isinstance(bar, ToolBarBreakWrapper):
                if bar.area is not None:
                    self.addToolBarBreak(bar.area)
                else:
                    self.addToolBarBreak()
            else:
                self.addToolBar(bar)

    def _set_dockwidgets(self, dockwidgets: QDockWidget) -> None:
        for dock in dockwidgets:
            self.addDockWidget(*[arg for arg in dock._args() if arg is not None])


from PySide6.QtWidgets import QPushButton, QToolButton
from PySide6.QtCore import Signal
from ..core import AbstractWidget


class _Button(AbstractWidget):
    long_press = Signal()
    long_repeat = Signal()
    long_release = Signal()
    long_click = Signal()

    def __init__(self, **kwargs):
        self._state = 0

        self.setAutoRepeat(True)
        self.setAutoRepeatDelay(1000)
        self.setAutoRepeatInterval(200)

        super().__init__(**kwargs)

        self.clicked.connect(self._handleLongClicked)

    def _handleLongClicked(self):
        if self.isDown():
            if self._state == 0:
                self._state = 1
                self.setAutoRepeatInterval(50)
                self.long_press.emit()
            else:
                self.long_repeat.emit()
        elif self._state == 1:
            self._state = 0
            self.setAutoRepeatInterval(200)
            self.long_release.emit()
        else:
            self.long_click.emit()


class PushButton(_Button, QPushButton):
    def __init__(self, **kwargs):
        QPushButton.__init__(self)
        super().__init__(**kwargs)


class ToolButton(_Button, QToolButton):
    def __init__(self, **kwargs):
        QToolButton.__init__(self)
        super().__init__(**kwargs)



# built-in
from .grid_layout import GridLayout
from .form_layout import FormLayout
from .h_layout import HLayout, ColumnLayout 
from .v_layout import VLayout, RowLayout
from .stacked import StackedLayout

# additional
from .border_layout import BorderLayout
from .flow_layout import FlowLayout


#!/usr/bin/python3

from PySide6.QtCore import Qt
from PySide6.QtWidgets import QWidget
from enum import IntFlag


class Alignment:
    Flag: IntFlag = Qt.AlignmentFlag

    def __init__(self,
                 *,
                 child: QWidget,
                 alignment: Qt.AlignmentFlag = Qt.AlignmentFlag.AlignCenter):

        self.child = child
        self.alignment = alignment


#!/usr/bin/python3

from typing import Any
from dataclasses import dataclass
from PySide6.QtCore import Qt
from PySide6.QtWidgets import QDockWidget


@dataclass
class DockWidgetWrapper:
    area: Qt.DockWidgetArea
    dockwidget: QDockWidget
    orient: Qt.Orientation = None

    def _args(self) -> list[Any]:
        return [self.area, self.dockwidget, self.orient]



#!/usr/bin/python3

from .api import Api
from ._base import AbstractBase, Args, MArgs, Func
from ._qobject import AbstractQObject, QObjectDec
from ._widget import AbstractWidget, WidgetDec
from ._tool import AbstractTool, ToolDec
from ._icons import AbstractIcons
from ._painter import AbstractPainter
from ._dialog import AbstractDialog
from ._config import AbstractConfig
from ._declarative import (
    DuplicateKeyError,
    TrackingDec,
    BehaviorDec,
    AbstractDec
)
from .routes import Routes
from .qstyle_sheet import QStyleSheet
from .exception_handler import TryExc


from typing import Union
from PySide6.QtWidgets import QSystemTrayIcon
from PySide6.QtGui import QIcon, QPixmap
from ..core import AbstractQObject


class SystemTray(AbstractQObject, QSystemTrayIcon):
    def __init__(self, **kwargs):
        QSystemTrayIcon.__init__(self)
        super().__init__(**kwargs)

    def alert(self, 
              title: str,
              msg: str = "", 
              icon: Union[QIcon, QPixmap, QSystemTrayIcon.MessageIcon] \
                  = QSystemTrayIcon.MessageIcon.Warning,
              timeout: int = 2000) -> None:
        self.showMessage(title, msg, icon, timeout)

    def notify(self, 
               title: str,
               msg: str = "", 
               icon: Union[QIcon, QPixmap, QSystemTrayIcon.MessageIcon] \
                   = QSystemTrayIcon.MessageIcon.Information,
               timeout: int = 2000) -> None:
        self.alert(title, msg, icon, timeout)


from typing import Union, List, Tuple, Callable, Any, TypeAlias 
from PySide6 import QtCore
from enum import Enum, auto


class Events(Enum):
    action: QtCore.QEvent = auto()
    change: QtCore.QEvent = auto()
    child: QtCore.QEvent = auto()
    close: QtCore.QEvent = auto()
    context_menu: QtCore.QEvent = auto()
    custom: QtCore.QEvent = auto()
    drag_enter: QtCore.QEvent = auto()
    drag_leave: QtCore.QEvent = auto()
    drag_move: QtCore.QEvent = auto()
    drop: QtCore.QEvent = auto()
    editor: QtCore.QEvent = auto()
    enter: QtCore.QEvent = auto()
    focus_in: QtCore.QEvent = auto()
    focus_out: QtCore.QEvent = auto()
    grab_keyboard: QtCore.QEvent = auto()
    grab_mouse: QtCore.QEvent = auto()
    help: QtCore.QEvent = auto()
    hide: QtCore.QEvent = auto()
    hover_enter: QtCore.QEvent = auto()
    hover_leave: QtCore.QEvent = auto()
    hover_move: QtCore.QEvent = auto()
    input_method: QtCore.QEvent = auto()
    key_press: QtCore.QEvent = auto()
    key_release: QtCore.QEvent = auto()
    leave: QtCore.QEvent = auto()
    mouse_double_click: QtCore.QEvent = auto()
    mouse_move: QtCore.QEvent = auto()
    mouse_press: QtCore.QEvent = auto()
    mouse_release: QtCore.QEvent = auto()
    move: QtCore.QEvent = auto()
    native: QtCore.QEvent = auto()
    paint: QtCore.QEvent = auto()
    polish: QtCore.QEvent = auto()
    resend_prepare: QtCore.QEvent = auto()
    resize: QtCore.QEvent = auto()
    scene: QtCore.QEvent = auto()
    send: QtCore.QEvent = auto()
    show: QtCore.QEvent = auto()
    tablet: QtCore.QEvent = auto()
    timer: QtCore.QEvent = auto()
    ungrab_keyboard: QtCore.QEvent = auto()
    ungrab_mouse: QtCore.QEvent = auto()
    viewport: QtCore.QEvent = auto()
    wheel: QtCore.QEvent = auto()
    widget: QtCore.QEvent = auto()
    window_frame: QtCore.QEvent = auto()

EventTypeVar: TypeAlias = List[Tuple[Union[Events, str], Callable[..., Any]]]



#!/usr/bin/python3

import math

from .geometry import circumcircle, minimum_bounding_circle, Vector
from .vector import Vector, Vector2D


def boundary(value, minvalue, maxvalue):
    '''Limit a value between a minvalue and maxvalue.'''
    return min(max(value, minvalue), maxvalue)


def intersection(set1, set2):
    '''Return the intersection of 2 lists.'''
    return [s for s in set1 if s in set2]


def difference(set1, set2):
    '''Return the difference between 2 lists.'''
    return [s for s in set1 if s not in set2]


def interpolate(value_from, value_to, step=10):
    '''Interpolate between two values. This can be useful for smoothing some
    transitions. For example::

        # instead of setting directly
        self.pos = pos

        # use interpolate, and you'll have a nicer transition
        self.pos = interpolate(self.pos, new_pos)

    .. warning::
        These interpolations work only on lists/tuples/doubles with the same
        dimensions. No test is done to check the dimensions are the same.
    '''
    if type(value_from) in (list, tuple):
        out = []
        for x, y in zip(value_from, value_to):
            out.append(interpolate(x, y, step))
        return out
    else:
        return value_from + (value_to - value_from) / float(step)

def format_bytes_to_human(size: int, precision: int = 2) -> str:
    '''Format a byte value to a human readable representation (B, KB, MB...).

    .. versionadded:: 1.0.8

    :Parameters:
        `size`: int
            Number that represents the bytes value
        `precision`: int, defaults to 2
            Precision after the comma

    Examples::

        >>> format_bytes_to_human(6463)
        '6.31 KB'
        >>> format_bytes_to_human(646368746541)
        '601.98 GB'
    '''
    size = int(size)
    fmt = '%%1.%df %%s' % precision
    for unit in ['B', 'KB', 'MB', 'GB', 'TB']:
        if size < 1024.0:
            return fmt % (size, unit)
        size /= 1024.0


def deg_to_coordinates(degrees: int, width: int) -> tuple[int, int, int, int]:
    # Convert degrees to radians
    radians = math.radians(degrees)

    # Calculate x and y components based on angle
    x_component = math.cos(radians)
    y_component = math.sin(radians)

    # Calculate half-width components
    half_width_x = (width / 2) * y_component
    half_width_y = (width / 2) * x_component

    # Define start and end points for a linear gradient
    xStart, yStart = (0.5 - half_width_x, 0.5 + half_width_y)
    xFinalStop, yFinalStop = (0.5 + half_width_x, 0.5 - half_width_y)

    return xStart, yStart, xFinalStop, yFinalStop


from typing import Union
from PySide6.QtGui import QPainter
from PySide6.QtCore import Qt, QEvent
from PySide6.QtWidgets import QGridLayout, QLayout, QWidget
from .tails import TeachingTipManager, _TailPos, _TailDirection
from ....core import AbstractDialog


class TeachingTipDialog(AbstractDialog):
    TailPos = _TailPos
    TailDirection = _TailDirection

    def __init__(self,
                 child: Union[QWidget, QLayout],
                 target: QWidget = None,
                 tail_pos: TailPos = TailPos.bottom,
                 tail_direction: TailDirection = TailDirection.center,
                 tail_len: int = 8,
                 auto_close: bool = True,
                 **kwargs) -> None:
        super().__init__(auto_close=auto_close, **kwargs)

        self._auto_close = auto_close

        self.child = child
        self.target = target

        self.manager = TeachingTipManager.make(tail_pos)
        self.manager.direction = tail_direction
        self.manager.lenght = tail_len

        self._layout = QGridLayout(self)
        self._layout.setContentsMargins(0, 0, 0, 0)
        self._layout.setSpacing(0)

        self.manager.doLayout(self)

        if isinstance(child, QLayout):
            self._layout.addLayout(child, 0, 0, Qt.AlignmentFlag.AlignCenter | Qt.AlignmentFlag.AlignHCenter)
        else:
            self._layout.addWidget(child)

        # set style
        self.setAttribute(Qt.WidgetAttribute.WA_TranslucentBackground)
        self.setAttribute(Qt.WidgetAttribute.WA_StyledBackground)
        self.setWindowFlags(
            Qt.WindowType.Tool
            | Qt.WindowType.FramelessWindowHint)

        if self.target is not None:
            self.target.resizeEvent = lambda e: self.update_pos()
            self.target.moveEvent = lambda e: self.update_pos()
            self.target.window().moveEvent = lambda e: self.update_pos()

    def tooltip_pos(self):
        return self.manager.position(self)

    def set_target(self, target: QWidget):
        self.target = target
        self.target.resizeEvent = lambda e: self.update_pos()
        self.target.moveEvent = lambda e: self.update_pos()
        self.target.window().moveEvent = lambda e: self.update_pos()

    def update_pos(self):
        self.move(self.manager.position(self))

    def showEvent(self, e):
        self.update_pos()
        super().showEvent(e)

    def eventFilter(self, obj, e: QEvent):
        if self.parent() and obj is self.parent().window():
            if e.type() in [QEvent.Type.Resize, 
                            QEvent.Type.WindowStateChange, 
                            QEvent.Type.Move]:
                self.move(self.manager.position(self))
        return super().eventFilter(obj, e)

    def paintEvent(self, e, scale: float = None):
        super().paintEvent(e)

        painter = QPainter(self)
        painter.setRenderHints(QPainter.RenderHint.Antialiasing)

        bg_color = self.child.palette().brush(self.child.backgroundRole())

        painter.setBrush(bg_color)
        painter.setPen(bg_color.color())

        self.manager.draw(self, painter)


#!/usr/bin/python3

from typing import Callable
from PySide6.QtWidgets import QWidget
from PySide6.QtCore import QRect, QTimer, Qt
from PySide6.QtGui import QColor, QPainter, QPen
from ..core import AbstractPainter


class CircularProgressPaint(AbstractPainter):
    def __init__(self,
                 *,
                 child: QWidget,
                 value: int = 0,
                 max_value: int = 100,
                 pen: QPen = None,
                 bg_color: QColor = QColor(0x44475a),
                 bg_enable: bool = False,
                 finished_callback: Callable = None,
                 changed_callback: Callable = None,
                 **kwargs):

        self._value = value
        self._max_value = max_value
        self._bg_enable = bg_enable
        self._bg_color = bg_color
        self._pen = pen

        self._finished_callback = finished_callback if finished_callback is not None else lambda: ...
        self._changed_callback = changed_callback if changed_callback is not None else lambda value: ...

        if not pen:
            self._pen = QPen()
            self._pen.setCapStyle(Qt.PenCapStyle.RoundCap)
            self._pen.setColor(QColor(0x498BD1))
            self._pen.setWidth(10)

        return super().__init__(child, **kwargs)

    def _update_callback(self):
        self._changed_callback(self.value)
        if self.value >= self.max_value:
            self._finished_callback()

    @property
    def value(self) -> int:
        return self._value

    @value.setter
    def value(self, value: int) -> None:
        self._value = value
        QTimer.singleShot(0, self._update_callback)
        self.repaint()

    @property
    def max_value(self) -> int:
        return self._max_value

    @max_value.setter
    def max_value(self, value: int) -> None:
        self._max_value = value
        self.repaint()

    def setValue(self, value: int):
        self.value = value
        QTimer.singleShot(0, self._update_callback)
        self.repaint()

    def setMaxValue(self, value: int):
        self.max_value = value
        self.repaint()

    def setPen(self, pen: QPen) -> None:
        self._pen = pen
        self.repaint()

    def paint(self, event):
        self.super_paintEvent(event)

        # Set Progress Parameters
        width = self._parent.width() - self._pen.width()
        height = self._parent.height() - self._pen.width()
        margin = self._pen.width() / 2
        value = self.value * 360 / self.max_value

        paint = QPainter()
        paint.begin(self._parent)
        paint.setRenderHint(QPainter.RenderHint.Antialiasing)

        # Create rectangle
        rect = QRect(0, 0, self._parent.width(), self._parent.height())
        paint.setPen(Qt.PenStyle.NoPen)
        paint.drawRect(rect)

        color = self._pen.color()
        if self._bg_enable:
            self._pen.setColor(QColor(0x44475a))
            paint.setPen(self._pen)
            paint.drawArc(margin, margin, width, height, 0, 360 * 16)

        # Create ARC / Circular progress
        self._pen.setColor(color)
        paint.setPen(self._pen)
        paint.drawArc(margin, margin, width, height, -90 * 16, -value * 16)
        paint.end()


#!/usr/bin/python3

from enum import Enum, auto
from PySide6.QtGui import (
    QBrush, 
    QIcon, 
    QImage, 
    QPaintEvent, 
    QPainter, 
    QPen, 
    QPicture, 
    QPixmap
)
from PySide6.QtCore import QPointF, QRectF, QSize, Qt
from PySide6.QtWidgets import QWidget
from ..core import AbstractPainter


class StatusEdgePaint(AbstractPainter):
    class Style(Enum):
        ellipse = auto()
        rectangle = auto()
        icon = auto()

    def __init__(self,
                 *,
                 child: QWidget,
                 corner: Qt.Corner = Qt.Corner.BottomRightCorner,
                 pen: QPen = None,
                 brush: QBrush = None,
                 padding: float = 1.5,
                 width: int = 13,
                 style: Style = Style.ellipse,
                 **kwargs) -> QWidget:

        self._width = width
        self._corner = corner
        self._padding = padding
        self._style = style
        self._icon = kwargs.get("icon", None)

        self._brush = brush
        self._pen = pen

        if not pen:
            self._pen = QPen()
            self._pen.setWidth(3)
            self._pen.setColor(0x151617)

        if not brush:
            self._brush = QBrush()
            self._brush.setColor(Qt.GlobalColor.white)

        return super().__init__(child, **kwargs)

    def set_pen(self, pen: QPen) -> None:
        self._pen = pen
        self.update()

    def set_brush(self, brush: QBrush) -> None:
        self._brush = brush
        self.update()

    def paint(self, event: QPaintEvent) -> None:
        self.super_paintEvent(event)

        painter = QPainter(self._parent)
        painter.setRenderHint(QPainter.RenderHint.Antialiasing)
        painter.setPen(self._pen)
        painter.setBrush(self._brush)

        if self._corner == Qt.Corner.BottomRightCorner:
            point = QPointF((self._parent.width() - self._width) - self._padding,
                            (self._parent.height() - self._width) - self._padding)

        elif self._corner == Qt.Corner.BottomLeftCorner:
            point = QPointF(self._padding, (self._parent.height() - self._width) - self._padding)

        elif self._corner == Qt.Corner.TopRightCorner:
            point = QPointF((self._parent.width() - self._width) - self._padding, self._padding)

        elif self._corner == Qt.Corner.TopLeftCorner:
            point = QPointF(self._padding, self._padding)

        if self._style == StatusEdgePaint.Style.ellipse:
            painter.drawEllipse(QRectF(point, QSize(self._width, self._width)))
        elif self._style == StatusEdgePaint.Style.rectangle:
            painter.drawRect(QRectF(point, QSize(self._width, self._width)))
        elif self._style == StatusEdgePaint.Style.icon and self._icon is not None:
            if isinstance(self._icon, QPicture):
                return painter.drawPicture(point, self._icon)
            elif isinstance(self._icon, QIcon):
                return self._icon.paint(painter, 
                                        QRectF(point, QSize(self._width, self._width)).toRect(), 
                                        Qt.AlignmentFlag.AlignCenter)
            elif isinstance(self._icon, QPixmap):
                func = painter.drawPixmap
            elif isinstance(self._icon, QImage):
                func = painter.drawImage

            func(QRectF(point, QSize(self._width, self._width)).toRect(), self._icon)